#!/usr/bin/env python

"""Tests for the ``extended_choices`` module.

Notes
-----

The documentation format in this file is numpydoc_.

.. _numpydoc: https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt

"""

from __future__ import unicode_literals

import sys
from copy import copy, deepcopy

try:
    import cPickle as pickle
except ImportError:
    import pickle

if sys.version_info >= (2, 7):
    import unittest
else:
    import unittest2 as unittest

import django

# Use an ordered dict, from python or django depending on the python version.
if sys.version_info >= (2, 7):
    from collections import OrderedDict
else:
    from django.utils.datastructures import SortedDict as OrderedDict

# Minimal django conf to test a real field.
from django.conf import settings
settings.configure(DATABASE_ENGINE='sqlite3')

from django.core.exceptions import ValidationError
from django.utils.functional import Promise
from django.utils.translation import ugettext_lazy

from .choices import Choices, OrderedChoices
from .fields import NamedExtendedChoiceFormField
from .helpers import ChoiceAttributeMixin, ChoiceEntry


class BaseTestCase(unittest.TestCase):
    """Base test case that define a test ``Choices`` instance with a subset."""

    def setUp(self):
        super(BaseTestCase, self).setUp()

        self.MY_CHOICES = None
        self.init_choices()

    def init_choices(self):

        self.MY_CHOICES = Choices(
            ('ONE', 1, 'One for the money'),
            ('TWO', 2, 'Two for the show'),
            ('THREE', 3, 'Three to get ready'),
        )
        self.MY_CHOICES.add_subset("ODD", ("ONE", "THREE"))


class FieldsTestCase(BaseTestCase):
    """Tests of the ``NamedExtendedChoiceFormField`` field."""

    def test_enforce_passing_a_choices(self):
        """Test that the field class only accepts a ``Choices`` object for its ``choices`` arg."""

        # Test with default choices tuple.
        with self.assertRaises(ValueError):
            NamedExtendedChoiceFormField(choices=(
                ('ONE', 1, 'One for the money'),
                ('TWO', 2, 'Two for the show'),
                ('THREE', 3, 'Three to get ready'),
            ))

        # Test with something else unrelated.
        with self.assertRaises(ValueError):
            NamedExtendedChoiceFormField(choices=1)

        # Test with a ``Choices`` object.
        field = NamedExtendedChoiceFormField(choices=self.MY_CHOICES)
        self.assertEqual(field.choices, self.MY_CHOICES)

    def test_named_extended_choice_form_field_validation(self):
        """Test that it only validation when it receives an existing constant name."""

        field = NamedExtendedChoiceFormField(choices=self.MY_CHOICES)

        # Should respect the constant case.
        self.assertEqual(field.clean('ONE'), 1)
        self.assertEqual(field.clean('TWO'), 2)

        with self.assertRaises(ValidationError) as raise_context:
            field.clean('one')
        self.assertEqual(raise_context.exception.code, 'non-existing-choice')

        # Should not validate with non-existing constant.
        with self.assertRaises(ValidationError) as raise_context:
            field.clean('FOUR')
        self.assertEqual(raise_context.exception.code, 'non-existing-choice')

        # Should fail early if not a string.
        with self.assertRaises(ValidationError) as raise_context:
            field.clean(1)
        self.assertEqual(raise_context.exception.code, 'invalid-choice-type')


class ChoicesTestCase(BaseTestCase):
    """Test the ``Choices`` class."""

    def test_should_behave_as_expected_by_django(self):
        """Test that it can be used by django, ie a list of tuple (value, display name)."""

        expected = (
            (1, 'One for the money'),
            (2, 'Two for the show'),
            (3, 'Three to get ready'),
        )

        # Test access to the whole expected tuple
        self.assertEqual(self.MY_CHOICES, expected)
        self.assertEqual(self.MY_CHOICES.choices, expected)

        # Test access to each tuple
        self.assertEqual(self.MY_CHOICES[0], expected[0])
        self.assertEqual(self.MY_CHOICES[2], expected[2])

    def test_should_be_accepted_by_django(self):
        """Test that a django field really accept a ``Choices`` instance."""

        from django.db.models import IntegerField
        field = IntegerField(choices=self.MY_CHOICES, default=self.MY_CHOICES.ONE)

        self.assertEqual(field.choices, self.MY_CHOICES)

        # No errors in ``_check_choices_``, Django 1.7+
        if django.VERSION >= (1, 7):
            self.assertEqual(field._check_choices(), [])

        # Test validation
        field.validate(1, None)

        with self.assertRaises(ValidationError) as raise_context:
            field.validate(4, None)

        # Check exception code, only in Django 1.6+
        if django.VERSION >= (1, 6):
            self.assertEqual(raise_context.exception.code, 'invalid_choice')

    def test_constants_attributes_should_return_values(self):
        """Test that each constant is an attribute returning the value."""

        self.assertEqual(self.MY_CHOICES.ONE, 1)
        self.assertEqual(self.MY_CHOICES.THREE, 3)

        with self.assertRaises(AttributeError):
            self.MY_CHOICES.FOUR

    def test_attributes_should_be_accessed_by_keys(self):
        """Test that each constant is accessible by key."""

        self.assertIs(self.MY_CHOICES['ONE'], self.MY_CHOICES.ONE)
        self.assertIs(self.MY_CHOICES['THREE'], self.MY_CHOICES.THREE)

        with self.assertRaises(KeyError):
            self.MY_CHOICES['FOUR']

    def test_entries(self):
        """Test that ``entries`` holds ``ChoiceEntry`` instances with correct attributes."""

        self.assertIsInstance(self.MY_CHOICES.entries[0], ChoiceEntry)
        self.assertEqual(self.MY_CHOICES.entries[0], ('ONE', 1, 'One for the money'))
        self.assertEqual(self.MY_CHOICES.entries[0].constant, 'ONE')
        self.assertEqual(self.MY_CHOICES.entries[0].value, 1)
        self.assertEqual(self.MY_CHOICES.entries[0].display, 'One for the money')

        self.assertIsInstance(self.MY_CHOICES.entries[2], ChoiceEntry)
        self.assertEqual(self.MY_CHOICES.entries[2], ('THREE', 3, 'Three to get ready'))
        self.assertEqual(self.MY_CHOICES.entries[2].constant, 'THREE')
        self.assertEqual(self.MY_CHOICES.entries[2].value, 3)
        self.assertEqual(self.MY_CHOICES.entries[2].display, 'Three to get ready')

    def test_dicts(self):
        """Test that ``constants``, ``values`` and ``displays`` dicts behave as expected."""
        self.assertIsInstance(self.MY_CHOICES.constants, dict)
        self.assertIsInstance(self.MY_CHOICES.values, dict)
        self.assertIsInstance(self.MY_CHOICES.displays, dict)

        self.assertDictEqual(self.MY_CHOICES.constants, {
            'ONE': ('ONE', 1, 'One for the money'),
            'TWO': ('TWO', 2, 'Two for the show'),
            'THREE': ('THREE', 3, 'Three to get ready'),
        })
        self.assertDictEqual(self.MY_CHOICES.values, {
            1: ('ONE', 1, 'One for the money'),
            2: ('TWO', 2, 'Two for the show'),
            3: ('THREE', 3, 'Three to get ready'),
        })
        self.assertDictEqual(self.MY_CHOICES.displays, {
            'One for the money': ('ONE', 1, 'One for the money'),
            'Two for the show': ('TWO', 2, 'Two for the show'),
            'Three to get ready': ('THREE', 3, 'Three to get ready'),
        })

    def test_adding_choices(self):
        """Test that we can add choices to an existing ``Choices`` instance."""
        self.MY_CHOICES.add_choices(
            ('FOUR', 4, 'And four to go'),
            ('FIVE', 5, '... but Five is not in the song'),
        )

        # Test django expected tuples
        expected = (
            (1, 'One for the money'),
            (2, 'Two for the show'),
            (3, 'Three to get ready'),
            (4, 'And four to go'),
            (5, '... but Five is not in the song'),
        )

        self.assertEqual(self.MY_CHOICES, expected)
        self.assertEqual(self.MY_CHOICES.choices, expected)

        # Test entries

        self.assertIsInstance(self.MY_CHOICES.entries[3], ChoiceEntry)
        self.assertEqual(self.MY_CHOICES.entries[3].constant, 'FOUR')
        self.assertEqual(self.MY_CHOICES.entries[3].value, 4)
        self.assertEqual(self.MY_CHOICES.entries[3].display, 'And four to go')

        self.assertIsInstance(self.MY_CHOICES.entries[4], ChoiceEntry)
        self.assertEqual(self.MY_CHOICES.entries[4].constant, 'FIVE')
        self.assertEqual(self.MY_CHOICES.entries[4].value, 5)
        self.assertEqual(self.MY_CHOICES.entries[4].display, '... but Five is not in the song')

        # Test dicts
        self.assertEqual(len(self.MY_CHOICES.constants), 5)
        self.assertEqual(len(self.MY_CHOICES.values), 5)
        self.assertEqual(len(self.MY_CHOICES.displays), 5)

        self.assertEqual(self.MY_CHOICES.constants['FOUR'],
                         ('FOUR', 4, 'And four to go'))
        self.assertEqual(self.MY_CHOICES.constants['FIVE'],
                         ('FIVE', 5, '... but Five is not in the song'))
        self.assertEqual(self.MY_CHOICES.values[4],
                         ('FOUR', 4, 'And four to go'))
        self.assertEqual(self.MY_CHOICES.values[5],
                         ('FIVE', 5, '... but Five is not in the song'))
        self.assertEqual(self.MY_CHOICES.displays['And four to go'],
                         ('FOUR', 4, 'And four to go'))
        self.assertEqual(self.MY_CHOICES.displays['... but Five is not in the song'],
                         ('FIVE', 5, '... but Five is not in the song'))

    def test_adding_choice_to_create_subset(self):
        """Test that we can create a subset while adding choices.

        This test test both ways of setting a name for a subset to create when adding choices,
        resetting ``MY_CHOICES`` between both tests, and calling ``test_subset`` in both cases.

        """

        def test_subset():
            # Quick check of the whole ``Choices`` (see ``test_adding_choices`` for full test)
            self.assertEqual(len(self.MY_CHOICES), 5)
            self.assertEqual(len(self.MY_CHOICES.entries), 5)
            self.assertEqual(len(self.MY_CHOICES.constants), 5)

            # Check that we have a subset
            self.assertIsInstance(self.MY_CHOICES.EXTENDED, Choices)

            # And that it contains the added choices (see ``test_creating_subset`` for full test)
            self.assertEqual(self.MY_CHOICES.EXTENDED, (
                (4, 'And four to go'),
                (5, '... but Five is not in the song'),
            ))

        # First test by setting the subset as first argument
        self.MY_CHOICES.add_choices(
            'EXTENDED',
            ('FOUR', 4, 'And four to go'),
            ('FIVE', 5, '... but Five is not in the song'),
        )

        test_subset()

        # Reset to remove added choices and subset
        self.init_choices()

        # Then test by setting the subset as named argument
        self.MY_CHOICES.add_choices(
            ('FOUR', 4, 'And four to go'),
            ('FIVE', 5, '... but Five is not in the song'),
            name='EXTENDED'
        )

        test_subset()

        # Using both first argument and a named argument should fail
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                'EXTENDED',
                ('FOUR', 4, 'And four to go'),
                ('FIVE', 5, '... but Five is not in the song'),
                name='EXTENDED'
            )

    def test_extracting_subset(self):
        """Test that we can extract a subset of choices."""

        subset = self.MY_CHOICES.extract_subset('ONE', 'TWO')

        self.assertIsInstance(subset, Choices)

        # Test django expected tuples
        expected = (
            (1, 'One for the money'),
            (2, 'Two for the show'),
        )

        self.assertEqual(subset, expected)
        self.assertEqual(subset.choices, expected)

        # Test entries
        self.assertEqual(len(subset.entries), 2)
        self.assertIsInstance(subset.entries[0], ChoiceEntry)
        self.assertEqual(subset.entries[0].constant, 'ONE')
        self.assertEqual(subset.entries[0].value, 1)
        self.assertEqual(subset.entries[0].display, 'One for the money')

        self.assertIsInstance(subset.entries[1], ChoiceEntry)
        self.assertEqual(subset.entries[1].constant, 'TWO')
        self.assertEqual(subset.entries[1].value, 2)
        self.assertEqual(subset.entries[1].display, 'Two for the show')

        # Test dicts
        self.assertEqual(len(subset.constants), 2)
        self.assertEqual(len(subset.values), 2)
        self.assertEqual(len(subset.displays), 2)

        self.assertIs(subset.constants['ONE'],
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(subset.constants['TWO'],
                      self.MY_CHOICES.constants['TWO'])
        self.assertIs(subset.values[1],
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(subset.values[2],
                      self.MY_CHOICES.constants['TWO'])
        self.assertIs(subset.displays['One for the money'],
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(subset.displays['Two for the show'],
                      self.MY_CHOICES.constants['TWO'])

        # Test ``in``
        self.assertIn(1, subset)
        self.assertNotIn(4, subset)

    def test_creating_subset(self):
        """Test that we can add subset of choices."""

        self.assertIsInstance(self.MY_CHOICES.ODD, Choices)

        # Test django expected tuples
        expected = (
            (1, 'One for the money'),
            (3, 'Three to get ready'),
        )

        self.assertEqual(self.MY_CHOICES.ODD, expected)
        self.assertEqual(self.MY_CHOICES.ODD.choices, expected)

        # Test entries
        self.assertEqual(len(self.MY_CHOICES.ODD.entries), 2)
        self.assertIsInstance(self.MY_CHOICES.ODD.entries[0], ChoiceEntry)
        self.assertEqual(self.MY_CHOICES.ODD.entries[0].constant, 'ONE')
        self.assertEqual(self.MY_CHOICES.ODD.entries[0].value, 1)
        self.assertEqual(self.MY_CHOICES.ODD.entries[0].display, 'One for the money')

        self.assertIsInstance(self.MY_CHOICES.ODD.entries[1], ChoiceEntry)
        self.assertEqual(self.MY_CHOICES.ODD.entries[1].constant, 'THREE')
        self.assertEqual(self.MY_CHOICES.ODD.entries[1].value, 3)
        self.assertEqual(self.MY_CHOICES.ODD.entries[1].display, 'Three to get ready')

        # Test dicts
        self.assertEqual(len(self.MY_CHOICES.ODD.constants), 2)
        self.assertEqual(len(self.MY_CHOICES.ODD.values), 2)
        self.assertEqual(len(self.MY_CHOICES.ODD.displays), 2)

        self.assertIs(self.MY_CHOICES.ODD.constants['ONE'],
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(self.MY_CHOICES.ODD.constants['THREE'],
                      self.MY_CHOICES.constants['THREE'])
        self.assertIs(self.MY_CHOICES.ODD.values[1],
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(self.MY_CHOICES.ODD.values[3],
                      self.MY_CHOICES.constants['THREE'])
        self.assertIs(self.MY_CHOICES.ODD.displays['One for the money'],
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(self.MY_CHOICES.ODD.displays['Three to get ready'],
                      self.MY_CHOICES.constants['THREE'])

        # Test ``in``
        self.assertIn(1, self.MY_CHOICES.ODD)
        self.assertNotIn(4, self.MY_CHOICES.ODD)

    def test_should_not_be_able_to_add_choices_to_a_subset(self):
        """Test that an exception is raised when trying to add new choices to a subset."""

        # Using a subset created by ``add_subset``.
        with self.assertRaises(RuntimeError):
            self.MY_CHOICES.ODD.add_choices(
                ('FOO', 10, 'foo'),
                ('BAR', 20, 'bar'),
            )

        # Using a subset created by ``add_choices``.
        self.MY_CHOICES.add_choices(
            ('FOUR', 4, 'And four to go'),
            ('FIVE', 5, '... but Five is not in the song'),
            name='EXTENDED'
        )
        with self.assertRaises(RuntimeError):
            self.MY_CHOICES.EXTENDED.add_choices(
                ('FOO', 10, 'foo'),
                ('BAR', 20, 'bar'),
            )

    def test_validating_added_choices(self):
        """Test that added choices can be added."""

        # Cannot add an existing constant.
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                ('ONE', 11, 'Another ONE'),
                ('FOUR', 4, 'And four to go'),
            )

        # Cannot add an existing value.
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                ('ONEBIS', 1, 'Another 1'),
                ('FOUR', 4, 'And four to go'),
            )

        # Cannot add two choices with the same name.
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                ('FOUR', 4, 'And four to go'),
                ('FOUR', 44, 'And four to go, bis'),
            )

        # Cannot add two choices with the same value.
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                ('FOUR', 4, 'And four to go'),
                ('FOURBIS', 4, 'And four to go, bis'),
            )

        # Cannot use existing attributes.
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                ('FOUR', 4, 'And four to go'),
                ('choices', 123, 'Existing attribute'),
            )

        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_choices(
                ('FOUR', 4, 'And four to go'),
                ('add_choices', 123, 'Existing attribute'),
            )

    def test_validating_subset(self):
        """Test that new subset is valid."""

        # Using a name that is already an attribute
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_subset("choices", ("ONE", "THREE"))

        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_subset("add_choices", ("ONE", "THREE"))

        # Using an undefined constant
        with self.assertRaises(ValueError):
            self.MY_CHOICES.add_subset("EVEN", ("TWO", "FOUR"))

    def test_for_methods(self):
        """Test the ``for_constant``, ``for_value`` and ``for_display`` methods."""

        self.assertIs(self.MY_CHOICES.for_constant('ONE'),
                      self.MY_CHOICES.constants['ONE'])
        self.assertIs(self.MY_CHOICES.for_value(2),
                      self.MY_CHOICES.values[2])
        self.assertIs(self.MY_CHOICES.for_display('Three to get ready'),
                      self.MY_CHOICES.displays['Three to get ready'])

        with self.assertRaises(KeyError):
            self.MY_CHOICES.for_constant('FOUR')

        with self.assertRaises(KeyError):
            self.MY_CHOICES.for_value(4)

        with self.assertRaises(KeyError):
            self.MY_CHOICES.for_display('And four to go')

    def test_has_methods(self):
        """Test the ``has_constant``, ``has_value`` and ``has_display`` methods."""

        self.assertTrue(self.MY_CHOICES.has_constant('ONE'))
        self.assertTrue(self.MY_CHOICES.has_value(2))
        self.assertTrue(self.MY_CHOICES.has_display('Three to get ready'))

        self.assertFalse(self.MY_CHOICES.has_constant('FOUR'))
        self.assertFalse(self.MY_CHOICES.has_value(4))
        self.assertFalse(self.MY_CHOICES.has_display('And four to go'))

    def test__contains__(self):
        """Test the ``__contains__`` method."""

        self.assertIn(self.MY_CHOICES.ONE, self.MY_CHOICES)
        self.assertTrue(self.MY_CHOICES.__contains__(self.MY_CHOICES.ONE))
        self.assertIn(1, self.MY_CHOICES)
        self.assertTrue(self.MY_CHOICES.__contains__(1))
        self.assertIn(3, self.MY_CHOICES)
        self.assertTrue(self.MY_CHOICES.__contains__(3))
        self.assertNotIn(4, self.MY_CHOICES)
        self.assertFalse(self.MY_CHOICES.__contains__(4))

    def test__getitem__(self):
        """Test the ``__getitem__`` method."""

        # Access to constants.
        self.assertEqual(self.MY_CHOICES['ONE'], 1)
        self.assertEqual(self.MY_CHOICES.__getitem__('ONE'), 1)

        self.assertEqual(self.MY_CHOICES['THREE'], 3)
        self.assertEqual(self.MY_CHOICES.__getitem__('THREE'), 3)

        with self.assertRaises(KeyError):
            self.MY_CHOICES['FOUR']
        with self.assertRaises(KeyError):
            self.MY_CHOICES.__getitem__('FOUR')

        one = (1, 'One for the money')
        three = (3, 'Three to get ready')

        # Access to default list entries
        self.assertEqual(self.MY_CHOICES[0], one)
        self.assertEqual(self.MY_CHOICES.__getitem__(0), one)

        self.assertEqual(self.MY_CHOICES[2], three)
        self.assertEqual(self.MY_CHOICES.__getitem__(2), three)

        with self.assertRaises(IndexError):
            self.MY_CHOICES[3]
        with self.assertRaises(IndexError):
            self.MY_CHOICES.__getitem__(3)

    def test_it_should_work_with_django_promises(self):
        """Test that it works with django promises, like ``ugettext_lazy``."""

        # Init django, only needed starting from django 1.7
        if django.VERSION >= (1, 7):
            django.setup()

        choices = Choices(
            ('ONE', 1, ugettext_lazy('one')),
            ('TWO', 2, ugettext_lazy('two')),
        )

        # Key in ``displays`` dict should be promises
        self.assertIsInstance(list(choices.displays.keys())[0], Promise)

        # And that they can be retrieved
        self.assertTrue(choices.has_display(ugettext_lazy('one')))
        self.assertEqual(choices.displays[ugettext_lazy('two')].value, 2)

        return

    def test_pickle_choice_attribute(self):
        """Test that a choice attribute could be pickled and unpickled."""

        value = self.MY_CHOICES.ONE

        pickled_value = pickle.dumps(value)
        unpickled_value = pickle.loads(pickled_value)

        self.assertEqual(unpickled_value, value)
        self.assertEqual(unpickled_value.choice_entry, value.choice_entry)
        self.assertEqual(unpickled_value.constant, 'ONE')
        self.assertEqual(unpickled_value.display, 'One for the money')
        self.assertEqual(unpickled_value.value, 1)

    def test_pickle_choice_entry(self):
        """Test that a choice entry could be pickled and unpickled."""

        entry = self.MY_CHOICES.ONE.choice_entry

        pickled_entry = pickle.dumps(entry)
        unpickled_entry = pickle.loads(pickled_entry)

        self.assertEqual(unpickled_entry, entry)
        self.assertEqual(unpickled_entry.constant, 'ONE')
        self.assertEqual(unpickled_entry.display, 'One for the money')
        self.assertEqual(unpickled_entry.value, 1)

    def test_pickle_choice(self):
        """Test that a choices object could be pickled and unpickled."""

        # Simple choice
        pickled_choices = pickle.dumps(self.MY_CHOICES)
        unpickled_choices = pickle.loads(pickled_choices)

        self.assertEqual(unpickled_choices, self.MY_CHOICES)

        # With a name, extra arguments and subsets
        OTHER_CHOICES = Choices(
            'ALL',
            ('ONE', 1, 'One for the money'),
            ('TWO', 2, 'Two for the show'),
            ('THREE', 3, 'Three to get ready'),
            dict_class=OrderedDict,
            mutable=False
        )
        OTHER_CHOICES.add_subset("ODD", ("ONE", "THREE"))
        OTHER_CHOICES.add_subset("EVEN", ("TWO", ))

        pickled_choices = pickle.dumps(OTHER_CHOICES)
        unpickled_choices = pickle.loads(pickled_choices)

        self.assertEqual(unpickled_choices, OTHER_CHOICES)
        self.assertEqual(unpickled_choices.dict_class, OrderedDict)
        self.assertFalse(unpickled_choices._mutable)
        self.assertEqual(unpickled_choices.subsets, OTHER_CHOICES.subsets)
        self.assertEqual(unpickled_choices.ALL, OTHER_CHOICES.ALL)
        self.assertEqual(unpickled_choices.ODD, OTHER_CHOICES.ODD)
        self.assertEqual(unpickled_choices.EVEN, OTHER_CHOICES.EVEN)

    def test_django_ugettext_lazy(self):
        """Test that a choices object using ugettext_lazy could be pickled and copied."""

        lazy_choices = Choices(
            ('ONE', 1, ugettext_lazy('One for the money')),
            ('TWO', 2, ugettext_lazy('Two for the show')),
            ('THREE', 3, ugettext_lazy('Three to get ready')),
        )

        # try to pickel it, it should not raise
        pickled_choices = pickle.dumps(lazy_choices)
        unpickled_choices = pickle.loads(pickled_choices)

        self.assertEqual(unpickled_choices, lazy_choices)

        # try to copy it, it should not raise
        copied_choices = copy(lazy_choices)
        self.assertEqual(copied_choices, lazy_choices)

        # try to deep-copy it, it should not raise
        deep_copied_choices = deepcopy(lazy_choices)
        self.assertEqual(deep_copied_choices, lazy_choices)

    def test_bool(self):
        """Test that having 0 or "" return `False` in a boolean context"""

        bool_choices = Choices(
            ('', 0, ''),
            ('FOO', 1, 'bar'),
        )

        first = bool_choices.for_value(0)
        second = bool_choices.for_value(1)

        self.assertFalse(first.constant)
        self.assertFalse(first.value)
        self.assertFalse(first.display)

        self.assertTrue(second.constant)
        self.assertTrue(second.value)
        self.assertTrue(second.display)

    def test_dict_class(self):
        """Test that the dict_class argument is taken into account"""

        dict_choices = Choices(
            ('FOO', 1, 'foo'),
            ('BAR', 2, 'bar')
        )

        self.assertIs(dict_choices.dict_class, dict)
        self.assertIsInstance(dict_choices.constants, dict)
        self.assertIsInstance(dict_choices.values, dict)
        self.assertIsInstance(dict_choices.displays, dict)

        ordered_dict_choices = Choices(
            ('FOO', 1, 'foo'),
            ('BAR', 2, 'bar'),
            dict_class=OrderedDict
        )

        self.assertIs(ordered_dict_choices.dict_class, OrderedDict)
        self.assertIsInstance(ordered_dict_choices.constants, OrderedDict)
        self.assertIsInstance(ordered_dict_choices.values, OrderedDict)
        self.assertIsInstance(ordered_dict_choices.displays, OrderedDict)

        ordered_choices = OrderedChoices(
            ('FOO', 1, 'foo'),
            ('BAR', 2, 'bar'),
        )

        self.assertIs(ordered_choices.dict_class, OrderedDict)
        self.assertIsInstance(ordered_choices.constants, OrderedDict)
        self.assertIsInstance(ordered_choices.values, OrderedDict)
        self.assertIsInstance(ordered_choices.displays, OrderedDict)


class ChoiceAttributeMixinTestCase(BaseTestCase):
    """Test the ``ChoiceAttributeMixin`` class."""

    choice_entry = ChoiceEntry(('FOO', 1, 'foo'))

    def test_it_should_respect_the_type(self):
        """A class based on ``ChoiceAttributeMixin`` should inherit from the other class too."""

        class StrChoiceAttribute(ChoiceAttributeMixin, str):
            pass

        str_attr = StrChoiceAttribute('f', self.choice_entry)
        self.assertEqual(str_attr, 'f')
        self.assertIsInstance(str_attr, str)

        class IntChoiceAttribute(ChoiceAttributeMixin, int):
            pass

        int_attr = IntChoiceAttribute(1, self.choice_entry)
        self.assertEqual(int_attr, 1)
        self.assertIsInstance(int_attr, int)

        class FloatChoiceAttribute(ChoiceAttributeMixin, float):
            pass

        float_attr = FloatChoiceAttribute(1.5, self.choice_entry)
        self.assertEqual(float_attr, 1.5)
        self.assertIsInstance(float_attr, float)

    def test_it_should_create_classes_on_the_fly(self):
        """Test that ``get_class_for_value works and cache its results."""

        # Empty list of cached classes to really test.
        ChoiceAttributeMixin._classes_by_type = {}

        # Create a class on the fly.
        IntClass = ChoiceAttributeMixin.get_class_for_value(1)
        self.assertEqual(IntClass.__name__, 'IntChoiceAttribute')
        self.assertIn(int, IntClass.mro())
        self.assertDictEqual(ChoiceAttributeMixin._classes_by_type, {
            int: IntClass
        })

        # Using the same type should return the same class as before.
        IntClass2 = ChoiceAttributeMixin.get_class_for_value(2)
        self.assertIs(IntClass2, IntClass2)

        int_attr = IntClass(1, self.choice_entry)
        self.assertEqual(int_attr, 1)
        self.assertIsInstance(int_attr, int)

        # Create another class on the fly.
        FloatClass = ChoiceAttributeMixin.get_class_for_value(1.5)
        self.assertEqual(FloatClass.__name__, 'FloatChoiceAttribute')
        self.assertIn(float, FloatClass.mro())
        self.assertDictEqual(ChoiceAttributeMixin._classes_by_type, {
            int: IntClass,
            float: FloatClass
        })

        float_attr = FloatClass(1.5, self.choice_entry)
        self.assertEqual(float_attr, 1.5)
        self.assertIsInstance(float_attr, float)

    def test_it_should_access_choice_entry_attributes(self):
        """Test that an instance can access the choice entry and its attributes."""
        IntClass = ChoiceAttributeMixin.get_class_for_value(1)
        attr = IntClass(1, self.choice_entry)

        # We should access the choice entry.
        self.assertEqual(attr.choice_entry, self.choice_entry)

        # And the attributes of the choice entry.
        self.assertEqual(attr.constant, self.choice_entry.constant)
        self.assertEqual(attr.value, self.choice_entry.value)
        self.assertEqual(attr.display, self.choice_entry.display)

    def test_it_should_work_with_django_promises(self):
        """Test that it works with django promises, like ``ugettext_lazy``."""

        # Init django, only needed starting from django 1.7
        if django.VERSION >= (1, 7):
            django.setup()

        value = ugettext_lazy('foo')
        klass = ChoiceAttributeMixin.get_class_for_value(value)
        attr = klass(value, self.choice_entry)

        self.assertIsInstance(attr, Promise)
        self.assertEqual(attr, ugettext_lazy('foo'))


class ChoiceEntryTestCase(BaseTestCase):
    """Test the ``ChoiceEntry`` class."""

    def test_it_should_act_as_a_normal_tuple(self):
        """Test that a ``ChoiceEntry`` instance acts as a real tuple."""

        # Test without additional attributes
        choice_entry = ChoiceEntry(('FOO', 1, 'foo'))
        self.assertIsInstance(choice_entry, tuple)
        self.assertEqual(choice_entry, ('FOO', 1, 'foo'))

        # Test with additional attributes
        choice_entry = ChoiceEntry(('FOO', 1, 'foo', {'bar': 1, 'baz': 2}))
        self.assertIsInstance(choice_entry, tuple)
        self.assertEqual(choice_entry, ('FOO', 1, 'foo'))

    def test_it_should_have_new_attributes(self):
        """Test that new specific attributes are accessible anv valid."""

        choice_entry = ChoiceEntry(('FOO', 1, 'foo'))

        self.assertEqual(choice_entry.constant, 'FOO')
        self.assertEqual(choice_entry.value, 1)
        self.assertEqual(choice_entry.display, 'foo')
        self.assertEqual(choice_entry.choice, (1, 'foo'))

    def test_new_attributes_are_instances_of_choices_attributes(self):
        """Test that the new attributes are instances of ``ChoiceAttributeMixin``."""

        choice_entry = ChoiceEntry(('FOO', 1, 'foo'))

        # 3 attributes should be choice attributes
        self.assertIsInstance(choice_entry.constant, ChoiceAttributeMixin)
        self.assertIsInstance(choice_entry.value, ChoiceAttributeMixin)
        self.assertIsInstance(choice_entry.display, ChoiceAttributeMixin)

        # As a choice attribute allow to access the other attributes of the choice entry,
        # check that it's really possible

        self.assertIs(choice_entry.constant.constant, choice_entry.constant)
        self.assertIs(choice_entry.constant.value, choice_entry.value)
        self.assertIs(choice_entry.constant.display, choice_entry.display)

        self.assertIs(choice_entry.value.constant, choice_entry.constant)
        self.assertIs(choice_entry.value.value, choice_entry.value)
        self.assertIs(choice_entry.value.display, choice_entry.display)

        self.assertIs(choice_entry.display.constant, choice_entry.constant)
        self.assertIs(choice_entry.display.value, choice_entry.value)
        self.assertIs(choice_entry.display.display, choice_entry.display)

        # Extreme test
        self.assertIs(choice_entry.display.value.constant.value.display.display.constant,
                      choice_entry.constant)

    def test_additional_attributes_are_saved(self):
        """Test that a dict passed as 4th tuple entry set its entries as attributes."""

        choice_entry = ChoiceEntry(('FOO', 1, 'foo', {'bar': 1, 'baz': 2}))
        self.assertEqual(choice_entry.bar, 1)
        self.assertEqual(choice_entry.baz, 2)

    def test_it_should_check_number_of_entries(self):
        """Test that it allows only tuples with 3 entries + optional attributes dict."""

        # Less than 3 shouldn't work
        with self.assertRaises(AssertionError):
            ChoiceEntry(('foo',))
        with self.assertRaises(AssertionError):
            ChoiceEntry((1, 'foo'))

        # More than 4 shouldn't work.
        with self.assertRaises(AssertionError):
            ChoiceEntry(('FOO', 1, 'foo', {'bar': 1, 'baz': 2}, 'QUZ'))
        with self.assertRaises(AssertionError):
            ChoiceEntry(('FOO', 1, 'foo', {'bar': 1, 'baz': 2}, 'QUZ', 'QUX'))

    def test_it_should_work_with_django_promises(self):
        """Test that ``ChoiceEntry`` class works with django promises, like ``ugettext_lazy``."""

        # Init django, only needed starting from django 1.7
        if django.VERSION >= (1, 7):
            django.setup()

        choice_entry = ChoiceEntry(('FOO', 1, ugettext_lazy('foo')))

        self.assertIsInstance(choice_entry.display, Promise)
        self.assertEqual(choice_entry.display, ugettext_lazy('foo'))

        self.assertEqual(choice_entry.display.constant, 'FOO')
        self.assertEqual(choice_entry.display.value, 1)
        self.assertEqual(choice_entry.display.display, ugettext_lazy('foo'))

    def test_it_should_raise_with_none_value(self):
        """Test that it's clear that we don't support None values."""

        with self.assertRaises(ValueError):
            ChoiceEntry(('FOO', None, 'foo'))


if __name__ == "__main__":
    unittest.main()



"""Provides classes used to construct a full ``Choices`` instance.

Notes
-----

The documentation format in this file is numpydoc_.

.. _numpydoc: https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt

"""

from __future__ import unicode_literals

from builtins import object  # pylint: disable=redefined-builtin

from django.utils.functional import Promise


class ChoiceAttributeMixin(object):
    """Base class to represent an attribute of a ``ChoiceEntry``.

    Used for ``constant``, ``name``, and ``display``.

    It must be used as a mixin with another type, and the final class will be a type with
    added attributes to access the ``ChoiceEntry`` instance and its attributes.

    Attributes
    ----------
    choice_entry : instance of ``ChoiceEntry``
        The ``ChoiceEntry`` instance that hold the current value, used to access its constant,
        value and display name.
    constant : property
        Returns the choice field holding the constant of the attached ``ChoiceEntry``.
    value : property
        Returns the choice field holding the value of the attached ``ChoiceEntry``.
    display : property
        Returns the choice field holding the display name of the attached ``ChoiceEntry``.
    original_value : ?
        The value used to create the current instance.
    creator_type : type
        The class that created a new class. Will be ``ChoiceAttributeMixin`` except if it was
        overridden by the author.

    Example
    -------

    Classes can be created manually:

    >>> class IntChoiceAttribute(ChoiceAttributeMixin, int): pass
    >>> field = IntChoiceAttribute(1, ChoiceEntry(('FOO', 1, 'foo')))
    >>> field
    1
    >>> field.constant, field.value, field.display
    ('FOO', 1, 'foo')
    >>> field.choice_entry
    ('FOO', 1, 'foo')

    Or via the ``get_class_for_value`` class method:

    >>> klass = ChoiceAttributeMixin.get_class_for_value(1.5)
    >>> klass.__name__
    'FloatChoiceAttribute'
    >>> float in klass.mro()
    True

    """

    def __new__(cls, *args, **kwargs):  # pylint: disable=unused-argument
        """Construct the object (the other class used with this mixin).

        Notes
        -----

        Only passes the very first argument to the ``super`` constructor.
        All others are not needed for the other class, only for this mixin.

        """

        if issubclass(cls, Promise):
            # Special case to manage lazy django stuff like ugettext_lazy
            return super(ChoiceAttributeMixin, cls).__new__(cls)

        return super(ChoiceAttributeMixin, cls).__new__(cls, *args[:1])

    def __init__(self, value, choice_entry):
        """Initiate the object to save the value and the choice entry.

        Parameters
        ----------
        value : ?
            Value to pass to the ``super`` constructor (for the other class using this mixin)
        choice_entry: ChoiceEntry
            The ``ChoiceEntry`` instance that hold the current value, used to access its constant,
            value and display name.

        Notes
        -----

        Call the ``super`` constructor with only the first value, as the other class doesn't
        expect the ``choice_entry`` parameter.

        """

        if isinstance(self, Promise):
            # Special case to manage lazy django stuff like ugettext_lazy
            # pylint: disable=protected-access
            super(ChoiceAttributeMixin, self).__init__(value._proxy____args, value._proxy____kw)
        else:
            super(ChoiceAttributeMixin, self).__init__()

        self.original_value = value
        self.choice_entry = choice_entry

    @property
    def constant(self):
        """Property that returns the ``constant`` attribute of the attached ``ChoiceEntry``."""
        return self.choice_entry.constant

    @property
    def value(self):
        """Property that returns the ``value`` attribute of the attached ``ChoiceEntry``."""
        return self.choice_entry.value

    @property
    def display(self):
        """Property that returns the ``display`` attribute of the attached ``ChoiceEntry``."""
        return self.choice_entry.display

    @classmethod
    def get_class_for_value(cls, value):
        """Class method to construct a class based on this mixin and the type of the given value.

        Parameters
        ----------
        value: ?
            The value from which to extract the type to create the new class.

        Notes
        -----
        The  create classes are cached (in ``cls.__classes_by_type``) to avoid recreating already
        created classes.
        """
        type_ = value.__class__

        # Check if the type is already a ``ChoiceAttribute``
        if issubclass(type_, ChoiceAttributeMixin):
            # In this case we can return this type
            return type_

        # Create a new class only if it wasn't already created for this type.
        if type_ not in cls._classes_by_type:
            # Compute the name of the class with the name of the type.
            class_name = str('%sChoiceAttribute' % type_.__name__.capitalize())
            # Create a new class and save it in the cache.
            cls._classes_by_type[type_] = type(class_name, (cls, type_), {
                'creator_type': cls,
            })

        # Return the class from the cache based on the type.
        return cls._classes_by_type[type_]

    def __reduce__(self):
        """Reducer to make the auto-created classes picklable.

        Returns
        -------
        tuple
            A tuple as expected by pickle, to recreate the object when calling ``pickle.loads``:
            1. a callable to recreate the object
            2. a tuple with all positioned arguments expected by this callable

        """

        return (
            # Function to create a choice attribute
            create_choice_attribute,
            (
                # The class that created the class of the current value
                self.creator_type,
                # The original type of the current value
                self.original_value,
                # The tied `choice_entry`
                self.choice_entry
            )
        )

    def __bool__(self):
        """Use the original value to know if the value is truthy of falsy"""
        return bool(self.original_value)

    _classes_by_type = {}


def create_choice_attribute(creator_type, value, choice_entry):
    """Create an instance of a subclass of ChoiceAttributeMixin for the given value.

    Parameters
    ----------
    creator_type : type
        ``ChoiceAttributeMixin`` or a subclass, from which we'll call the ``get_class_for_value``
        class-method.
    value : ?
        The value for which we want to create an instance of a new subclass of ``creator_type``.
    choice_entry: ChoiceEntry
        The ``ChoiceEntry`` instance that hold the current value, used to access its constant,
        value and display name.

    Returns
    -------
    ChoiceAttributeMixin
        An instance of a subclass of ``creator_type`` for the given value

    """

    klass = creator_type.get_class_for_value(value)
    return klass(value, choice_entry)


class ChoiceEntry(tuple):
    """Represents a choice in a ``Choices`` object, with easy access to its attribute.

    Expecting a tuple with three entries. (constant, value, display name), it will add three
    attributes to access then: ``constant``, ``value`` and ``display``.

    By passing a dict after these three first entries, in the tuple, it's also possible to
    add some other attributes to the ``ChoiceEntry` instance``.

    Parameters
    ----------
    tuple_ : tuple
        A tuple with three entries, the name of the constant, the value, and the display name.
        A dict could be added as a fourth entry to add additional attributes.


    Example
    -------

    >>> entry = ChoiceEntry(('FOO', 1, 'foo'))
    >>> entry
    ('FOO', 1, 'foo')
    >>> (entry.constant, entry.value, entry.display)
    ('FOO', 1, 'foo')
    >>> entry.choice
    (1, 'foo')

    You can also pass attributes to add to the instance to create:

    >>> entry = ChoiceEntry(('FOO', 1, 'foo', {'bar': 1, 'baz': 2}))
    >>> entry
    ('FOO', 1, 'foo')
    >>> entry.bar
    1
    >>> entry.baz
    2

    Raises
    ------
    AssertionError
        If the number of entries in the tuple is not expected. Must be 3 or 4.

    """

    # Allow to easily change the mixin to use in subclasses.
    ChoiceAttributeMixin = ChoiceAttributeMixin

    def __new__(cls, tuple_):
        """Construct the tuple with 3 entries, and save optional attributes from the 4th one."""

        # Ensure we have exactly 3 entries in the tuple and an optional dict.
        assert 3 <= len(tuple_) <= 4, 'Invalid number of entries in %s' % (tuple_,)

        # Call the ``tuple`` constructor with only the real tuple entries.
        obj = super(ChoiceEntry, cls).__new__(cls, tuple_[:3])

        # Save all special attributes.
        # pylint: disable=protected-access
        obj.constant = obj._get_choice_attribute(tuple_[0])
        obj.value = obj._get_choice_attribute(tuple_[1])
        obj.display = obj._get_choice_attribute(tuple_[2])

        # Add an attribute holding values as expected by django.
        obj.choice = (obj.value, obj.display)

        # Add additional attributes.
        if len(tuple_) == 4:
            for key, value in tuple_[3].items():
                setattr(obj, key, value)

        return obj

    def _get_choice_attribute(self, value):
        """Get a choice attribute for the given value.

        Parameters
        ----------
        value: ?
            The value for which we want a choice attribute.

        Returns
        -------
        An instance of a class based on ``ChoiceAttributeMixin`` for the given value.

        Raises
        ------
        ValueError
            If the value is None, as we cannot really subclass NoneType.

        """

        if value is None:
            raise ValueError('Using `None` in a `Choices` object is not supported. You may '
                             'use an empty string.')

        return create_choice_attribute(self.ChoiceAttributeMixin, value, self)



"""Little helper application to improve django choices (for fields)"""

from __future__ import unicode_literals

from .choices import Choices, OrderedChoices

__author__ = 'Stephane "Twidi" Ange;'
__contact__ = "s.angel@twidi.com"
__homepage__ = "https://pypi.python.org/pypi/django-extended-choices"
__version__ = "1.1.1"



"""Provides a form field for django to use constants instead of values as available values.

Notes
-----

The documentation format in this file is `numpydoc`_.

.. _numpydoc: https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt

"""

from __future__ import unicode_literals

from past.builtins import basestring

from django import forms

from . import Choices


class NamedExtendedChoiceFormField(forms.Field):
    """Field to use with choices where values are constant names instead of choice values.

    Should not be very useful in normal HTML form, but if API validation is done via a form, it
    will to have more readable constants in the API that values
    """
    def __init__(self, choices, *args, **kwargs):
        """Override to ensure that the ``choices`` argument is a ``Choices`` object."""

        super(NamedExtendedChoiceFormField, self).__init__(*args, **kwargs)

        if not isinstance(choices, Choices):
            raise ValueError("`choices` must be an instance of `extended_choices.Choices`.")

        self.choices = choices

    def to_python(self, value):
        """Convert the constant to the real choice value."""

        # ``is_required`` is already checked in ``validate``.
        if value is None:
            return None

        # Validate the type.
        if not isinstance(value, basestring):
            raise forms.ValidationError(
                "Invalid value type (should be a string).",
                code='invalid-choice-type',
            )

        # Get the constant from the choices object, raising if it doesn't exist.
        try:
            final = getattr(self.choices, value)
        except AttributeError:
            available = '[%s]' % ', '.join(self.choices.constants)
            raise forms.ValidationError(
                "Invalid value (not in available choices. Available ones are: %s" % available,
                code='non-existing-choice',
            )

        return final



"""Provides a ``Choices`` class to help using "choices" in Django fields.

The aim is to replace:

.. code-block:: python

    STATE_ONLINE  = 1
    STATE_DRAFT   = 2
    STATE_OFFLINE = 3

    STATE_CHOICES = (
        (STATE_ONLINE,  'Online'),
        (STATE_DRAFT,   'Draft'),
        (STATE_OFFLINE, 'Offline'),
    )

    STATE_DICT = dict(STATE_CHOICES)

    class Content(models.Model):
        title      = models.CharField(max_length=255)
        content    = models.TextField()
        state      = models.PositiveSmallIntegerField(choices=STATE_CHOICES, default=STATE_DRAFT)

        def __unicode__(self):
            return 'Content "%s" (state=%s)' % (self.title, STATE_DICT[self.state])

    print(Content.objects.filter(state=STATE_ONLINE))

By this:

.. code-block:: python

    from extended_choices import Choices

    STATES = Choices(
        ('ONLINE',  1, 'Online'),
        ('DRAFT',   2, 'Draft'),
        ('OFFLINE', 3, 'Offline'),
    )

    class Content(models.Model):
        title      = models.CharField(max_length=255)
        content    = models.TextField()
        state      = models.PositiveSmallIntegerField(choices=STATES, default=STATES.DRAFT)

        def __unicode__(self):
            return 'Content "%s" (state=%s)' % (self.title, STATES.for_value(self.state).display)

    print(Content.objects.filter(state=STATES.ONLINE))


Notes
-----

The documentation format in this file is numpydoc_.

.. _numpydoc: https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt

"""

from __future__ import unicode_literals

import sys
from past.builtins import basestring

if sys.version_info >= (2, 7):
    from collections import OrderedDict
else:
    from django.utils.datastructures import SortedDict as OrderedDict

from .helpers import ChoiceEntry

__all__ = ['Choices']


class Choices(list):
    """Helper class for choices fields in Django

    A choice entry has three representation: constant name, value and
    display name). So ``Choices`` takes list of such tuples.

    It's easy to get the constant, value or display name given one of these value. See in
    example.

    Parameters
    ----------
    *choices : list of tuples
        It's the list of tuples to add to the ``Choices`` instance, each tuple having three
        entries: the constant name, the value, the display name.

        A dict could be added as a 4th entry in the tuple to allow setting arbitrary
        arguments to the final ``ChoiceEntry`` created for this choice tuple.

    name : string, optional
        If set, a subset will be created containing all the constants. It could be used if you
        construct your ``Choices`` instance with many calls to ``add_choices``.
    dict_class : type, optional
        ``dict`` by default, it's the dict class to use to create dictionaries (``constants``,
        ``values`` and ``displays``. Could be set for example to ``OrderedDict`` (you can use
        ``OrderedChoices`` that is a simple subclass using ``OrderedDict``.

    Example
    -------

    Start by declaring your ``Choices``:

    >>> ALIGNMENTS = Choices(
    ...     ('BAD', 10, 'bad'),
    ...     ('NEUTRAL', 20, 'neutral'),
    ...     ('CHAOTIC_GOOD', 30, 'chaotic good'),
    ...     ('GOOD', 40, 'good'),
    ...     dict_class=OrderedDict
    ... )

    Then you can use it in a django field, Notice its usage in ``choices`` and ``default``:

    >>> from django.conf import settings
    >>> try:
    ...     settings.configure(DATABASE_ENGINE='sqlite3')
    ... except: pass
    >>> from django.db.models import IntegerField
    >>> field = IntegerField(choices=ALIGNMENTS,  # use ``ALIGNMENTS`` or ``ALIGNMENTS.choices``.
    ...                      default=ALIGNMENTS.NEUTRAL)

    The ``Choices`` returns a list as expected by django:

    >>> ALIGNMENTS == ((10, 'bad'), (20, 'neutral'), (30, 'chaotic good'), (40, 'good'))
    True

    But represents it with the constants:

    >>> repr(ALIGNMENTS)
    "[('BAD', 10, 'bad'), ('NEUTRAL', 20, 'neutral'), ('CHAOTIC_GOOD', 30, 'chaotic good'), ('GOOD', 40, 'good')]"

    Use ``choices`` which is a simple list to represent it as such:

    >>> ALIGNMENTS.choices
    ((10, 'bad'), (20, 'neutral'), (30, 'chaotic good'), (40, 'good'))


    And you can access value by their constant, or as you want:

    >>> ALIGNMENTS.BAD
    10
    >>> ALIGNMENTS.BAD.display
    'bad'
    >>> 40 in ALIGNMENTS
    True
    >>> ALIGNMENTS.has_constant('BAD')
    True
    >>> ALIGNMENTS.has_value(20)
    True
    >>> ALIGNMENTS.has_display('good')
    True
    >>> ALIGNMENTS.for_value(10)
    ('BAD', 10, 'bad')
    >>> ALIGNMENTS.for_value(10).constant
    'BAD'
    >>> ALIGNMENTS.for_display('good').value
    40
    >>> ALIGNMENTS.for_constant('NEUTRAL').display
    'neutral'
    >>> ALIGNMENTS.constants
    OrderedDict([('BAD', ('BAD', 10, 'bad')), ('NEUTRAL', ('NEUTRAL', 20, 'neutral')), ('CHAOTIC_GOOD', ('CHAOTIC_GOOD', 30, 'chaotic good')), ('GOOD', ('GOOD', 40, 'good'))])
    >>> ALIGNMENTS.values
    OrderedDict([(10, ('BAD', 10, 'bad')), (20, ('NEUTRAL', 20, 'neutral')), (30, ('CHAOTIC_GOOD', 30, 'chaotic good')), (40, ('GOOD', 40, 'good'))])
    >>> ALIGNMENTS.displays
    OrderedDict([('bad', ('BAD', 10, 'bad')), ('neutral', ('NEUTRAL', 20, 'neutral')), ('chaotic good', ('CHAOTIC_GOOD', 30, 'chaotic good')), ('good', ('GOOD', 40, 'good'))])

    You can create subsets of choices:

    >>> ALIGNMENTS.add_subset('WESTERN',('BAD', 'GOOD'))
    >>> ALIGNMENTS.WESTERN.choices
    ((10, 'bad'), (40, 'good'))
    >>> ALIGNMENTS.BAD in ALIGNMENTS.WESTERN
    True
    >>> ALIGNMENTS.NEUTRAL in ALIGNMENTS.WESTERN
    False

    To use it in another field (only the values in the subset will be available), or for checks:

    >>> def is_western(value):
    ...     return value in ALIGNMENTS.WESTERN
    >>> is_western(40)
    True

    """

    # Allow to easily change the ``ChoiceEntry`` class to use in subclasses.
    ChoiceEntryClass = ChoiceEntry

    def __init__(self, *choices, **kwargs):

        # Init the list as empty. Entries will be formatted for django and added in
        # ``add_choices``.
        super(Choices, self).__init__()

        # Class to use for dicts.
        self.dict_class = kwargs.get('dict_class', dict)

        # List of ``ChoiceEntry``, one for each choice in this instance.
        self.entries = []

        # List of the created subsets
        self.subsets = []

        # Dicts to access ``ChoiceEntry`` instances by constant, value or display value.
        self.constants = self.dict_class()
        self.values = self.dict_class()
        self.displays = self.dict_class()

        # For now this instance is mutable: we need to add the given choices.
        self._mutable = True
        self.add_choices(*choices, name=kwargs.get('name', None))

        # Now we can set ``_mutable`` to its correct value.
        self._mutable = kwargs.get('mutable', True)

    @property
    def choices(self):
        """Property that returns a tuple formatted as expected by Django.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.choices
        ((1, 'foo'), (2, 'bar'))

        """
        return tuple(self)

    def add_choices(self, *choices, **kwargs):
        """Add some choices to the current ``Choices`` instance.

        The given choices will be added to the existing choices.
        If a ``name`` attribute is passed, a new subset will be created with all the given
        choices.

        Note that it's not possible to add new choices to a subset.

        Parameters
        ----------
        *choices : list of tuples
            It's the list of tuples to add to the ``Choices`` instance, each tuple having three
            entries: the constant name, the value, the display name.

            A dict could be added as a 4th entry in the tuple to allow setting arbitrary
            arguments to the final ``ChoiceEntry`` created for this choice tuple.

            If the first entry of ``*choices`` is a string, then it will be used as a name for a
            new subset that will contain all the given choices.
        **kwargs : dict
            name : string
                Instead of using the first entry of the ``*choices`` to pass a name of a subset to
                create, you can pass it via the ``name`` named argument.

        Example
        -------

        >>> MY_CHOICES = Choices()
        >>> MY_CHOICES.add_choices(('ZERO', 0, 'zero'))
        >>> MY_CHOICES
        [('ZERO', 0, 'zero')]
        >>> MY_CHOICES.add_choices('SMALL', ('ONE', 1, 'one'), ('TWO', 2, 'two'))
        >>> MY_CHOICES
        [('ZERO', 0, 'zero'), ('ONE', 1, 'one'), ('TWO', 2, 'two')]
        >>> MY_CHOICES.SMALL
        [('ONE', 1, 'one'), ('TWO', 2, 'two')]
        >>> MY_CHOICES.add_choices(('THREE', 3, 'three'), ('FOUR', 4, 'four'), name='BIG')
        >>> MY_CHOICES
        [('ZERO', 0, 'zero'), ('ONE', 1, 'one'), ('TWO', 2, 'two'), ('THREE', 3, 'three'), ('FOUR', 4, 'four')]
        >>> MY_CHOICES.BIG
        [('THREE', 3, 'three'), ('FOUR', 4, 'four')]

        Raises
        ------
        RuntimeError
            When the ``Choices`` instance is marked as not mutable, which is the case for subsets.

        ValueError

            * if the subset name is defined as first argument and as named argument.
            * if some constants have the same name or the same value.
            * if at least one constant or value already exists in the instance.

        """

        # It the ``_mutable`` flag is falsy, which is the case for subsets, we refuse to add
        # new choices.
        if not self._mutable:
            raise RuntimeError("This ``Choices`` instance cannot be updated.")

        # Check for an optional subset name as the first argument (so the first entry of *choices).
        subset_name = None
        if choices and isinstance(choices[0], basestring):
            subset_name = choices[0]
            choices = choices[1:]

        # Check for an optional subset name in the named arguments.
        if kwargs.get('name', None):
            if subset_name:
                raise ValueError("The name of the subset cannot be defined as the first "
                                 "argument and also as a named argument")
            subset_name = kwargs['name']

        # Check that each new constant is unique.
        constants = [c[0] for c in choices]
        constants_doubles = [c for c in constants if constants.count(c) > 1]
        if constants_doubles:
            raise ValueError("You cannot declare two constants with the same constant name. "
                             "Problematic constants: %s " % list(set(constants_doubles)))

        # Check that none of the new constants already exists.
        bad_constants = set(constants).intersection(self.constants)
        if bad_constants:
            raise ValueError("You cannot add existing constants. "
                             "Existing constants: %s." % list(bad_constants))

        # Check that none of the constant is an existing attributes
        bad_constants = [c for c in constants if hasattr(self, c)]
        if bad_constants:
            raise ValueError("You cannot add constants that already exists as attributes. "
                             "Existing attributes: %s." % list(bad_constants))

        # Check that each new value is unique.
        values = [c[1] for c in choices]
        values_doubles = [c for c in values if values.count(c) > 1]
        if values_doubles:
            raise ValueError("You cannot declare two choices with the same name."
                             "Problematic values: %s " % list(set(values_doubles)))

        # Check that none of the new values already exists.
        bad_values = set(values).intersection(self.values)
        if bad_values:
            raise ValueError("You cannot add existing values. "
                             "Existing values: %s." % list(bad_values))

        # We can now add each choice.
        for choice_tuple in choices:

            # Convert the choice tuple in a ``ChoiceEntry`` instance if it's not already done.
            # It allows to share choice entries between a ``Choices`` instance and its subsets.
            choice_entry = choice_tuple
            if not isinstance(choice_entry, self.ChoiceEntryClass):
                choice_entry = self.ChoiceEntryClass(choice_entry)

            # Append to the main list the choice as expected by django: (value, display name).
            self.append(choice_entry.choice)
            # And the ``ChoiceEntry`` instance to our own internal list.
            self.entries.append(choice_entry)

            # Make the value accessible via an attribute (the constant being its name).
            setattr(self, choice_entry.constant, choice_entry.value)

            # Fill dicts to access the ``ChoiceEntry`` instance by its constant, value or display..
            self.constants[choice_entry.constant] = choice_entry
            self.values[choice_entry.value] = choice_entry
            self.displays[choice_entry.display] = choice_entry

        # If we have a subset name, create a new subset with all the given constants.
        if subset_name:
            self.add_subset(subset_name, constants)

    def extract_subset(self, *constants):
        """Create a subset of entries

        This subset is a new ``Choices`` instance, with only the wanted constants from the
        main ``Choices`` (each "choice entry" in the subset is shared from the main ``Choices``)

        Parameters
        ----------
        *constants: list
            The constants names of this ``Choices`` object to make available in the subset.

        Returns
        -------
        Choices
            The newly created subset, which is a ``Choices`` object


        Example
        -------

        >>> STATES = Choices(
        ...     ('ONLINE',  1, 'Online'),
        ...     ('DRAFT',   2, 'Draft'),
        ...     ('OFFLINE', 3, 'Offline'),
        ... )
        >>> STATES
        [('ONLINE', 1, 'Online'), ('DRAFT', 2, 'Draft'), ('OFFLINE', 3, 'Offline')]
        >>> subset = STATES.extract_subset('DRAFT', 'OFFLINE')
        >>> subset
        [('DRAFT', 2, 'Draft'), ('OFFLINE', 3, 'Offline')]
        >>> subset.DRAFT
        2
        >>> subset.for_constant('DRAFT') is STATES.for_constant('DRAFT')
        True
        >>> subset.ONLINE
        Traceback (most recent call last):
        ...
        AttributeError: 'Choices' object has no attribute 'ONLINE'


        Raises
        ------
        ValueError
            If a constant is not defined as a constant in the ``Choices`` instance.

        """

        # Ensure that all passed constants exists as such in the list of available constants.
        bad_constants = set(constants).difference(self.constants)
        if bad_constants:
            raise ValueError("All constants in subsets should be in parent choice. "
                             "Missing constants: %s." % list(bad_constants))

        # Keep only entries we asked for.
        choice_entries = [self.constants[c] for c in constants]

        # Create a new ``Choices`` instance with the limited set of entries, and pass the other
        # configuration attributes to share the same behavior as the current ``Choices``.
        # Also we set ``mutable`` to False to disable the possibility to add new choices to the
        # subset.
        subset = self.__class__(
            *choice_entries,
            **{
                'dict_class': self.dict_class,
                'mutable': False,
            }
        )

        return subset

    def add_subset(self, name, constants):
        """Add a subset of entries under a defined name.

        This allow to defined a "sub choice" if a django field need to not have the whole
        choice available.

        The sub-choice is a new ``Choices`` instance, with only the wanted the constant from the
        main ``Choices`` (each "choice entry" in the subset is shared from the main ``Choices``)
        The sub-choice is accessible from the main ``Choices`` by an attribute having the given
        name.

        Parameters
        ----------
        name : string
            Name of the attribute that will old the new ``Choices`` instance.
        constants: list or tuple
            List of the constants name of this ``Choices`` object to make available in the subset.


        Returns
        -------
        Choices
            The newly created subset, which is a ``Choices`` object


        Example
        -------

        >>> STATES = Choices(
        ...     ('ONLINE',  1, 'Online'),
        ...     ('DRAFT',   2, 'Draft'),
        ...     ('OFFLINE', 3, 'Offline'),
        ... )
        >>> STATES
        [('ONLINE', 1, 'Online'), ('DRAFT', 2, 'Draft'), ('OFFLINE', 3, 'Offline')]
        >>> STATES.add_subset('NOT_ONLINE', ('DRAFT', 'OFFLINE',))
        >>> STATES.NOT_ONLINE
        [('DRAFT', 2, 'Draft'), ('OFFLINE', 3, 'Offline')]
        >>> STATES.NOT_ONLINE.DRAFT
        2
        >>> STATES.NOT_ONLINE.for_constant('DRAFT') is STATES.for_constant('DRAFT')
        True
        >>> STATES.NOT_ONLINE.ONLINE
        Traceback (most recent call last):
        ...
        AttributeError: 'Choices' object has no attribute 'ONLINE'


        Raises
        ------
        ValueError

            * If ``name`` is already an attribute of the ``Choices`` instance.
            * If a constant is not defined as a constant in the ``Choices`` instance.

        """

        # Ensure that the name is not already used as an attribute.
        if hasattr(self, name):
            raise ValueError("Cannot use '%s' as a subset name. "
                             "It's already an attribute." % name)

        subset = self.extract_subset(*constants)

        # Make the subset accessible via an attribute.
        setattr(self, name, subset)
        self.subsets.append(name)

    def for_constant(self, constant):
        """Returns the ``ChoiceEntry`` for the given constant.

        Parameters
        ----------
        constant: string
            Name of the constant for which we want the choice entry.

        Returns
        -------
        ChoiceEntry
            The instance of ``ChoiceEntry`` for the given constant.

        Raises
        ------
        KeyError
            If the constant is not an existing one.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.for_constant('FOO')
        ('FOO', 1, 'foo')
        >>> MY_CHOICES.for_constant('FOO').value
        1
        >>> MY_CHOICES.for_constant('QUX')
        Traceback (most recent call last):
        ...
        KeyError: 'QUX'

        """

        return self.constants[constant]

    def for_value(self, value):
        """Returns the ``ChoiceEntry`` for the given value.

        Parameters
        ----------
        value: ?
            Value for which we want the choice entry.

        Returns
        -------
        ChoiceEntry
            The instance of ``ChoiceEntry`` for the given value.

        Raises
        ------
        KeyError
            If the value is not an existing one.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.for_value(1)
        ('FOO', 1, 'foo')
        >>> MY_CHOICES.for_value(1).display
        'foo'
        >>> MY_CHOICES.for_value(3)
        Traceback (most recent call last):
        ...
        KeyError: 3

        """

        return self.values[value]

    def for_display(self, display):
        """Returns the ``ChoiceEntry`` for the given display name.

        Parameters
        ----------
        display: string
            Display name for which we want the choice entry.

        Returns
        -------
        ChoiceEntry
            The instance of ``ChoiceEntry`` for the given display name.

        Raises
        ------
        KeyError
            If the display name is not an existing one.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.for_display('foo')
        ('FOO', 1, 'foo')
        >>> MY_CHOICES.for_display('foo').constant
        'FOO'
        >>> MY_CHOICES.for_display('qux')
        Traceback (most recent call last):
        ...
        KeyError: 'qux'

        """

        return self.displays[display]

    def has_constant(self, constant):
        """Check if the current ``Choices`` object has the given constant.

        Parameters
        ----------
        constant: string
            Name of the constant we want to check..

        Returns
        -------
        boolean
            ``True`` if the constant is present, ``False`` otherwise.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.has_constant('FOO')
        True
        >>> MY_CHOICES.has_constant('QUX')
        False

        """

        return constant in self.constants

    def has_value(self, value):
        """Check if the current ``Choices`` object has the given value.

        Parameters
        ----------
        value: ?
            Value we want to check.

        Returns
        -------
        boolean
            ``True`` if the value is present, ``False`` otherwise.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.has_value(1)
        True
        >>> MY_CHOICES.has_value(3)
        False

        """

        return value in self.values

    def has_display(self, display):
        """Check if the current ``Choices`` object has the given display name.

        Parameters
        ----------
        display: string
            Display name we want to check..

        Returns
        -------
        boolean
            ``True`` if the display name is present, ``False`` otherwise.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES.has_display('foo')
        True
        >>> MY_CHOICES.has_display('qux')
        False

        """

        return display in self.displays

    def __contains__(self, item):
        """Check if the current ``Choices`` object has the given value.

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> 1 in MY_CHOICES
        True
        >>> 3 in MY_CHOICES
        False

        """

        return self.has_value(item)

    def __getitem__(self, key):
        """Return the attribute having the given name for the current instance

        It allows for example to retrieve constant by keys instead of by attributes (as constants
        are set as attributes to easily get the matching value.)

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES['FOO']
        1
        >>> MY_CHOICES['constants'] is MY_CHOICES.constants
        True

        """

        # If the key is an int, call ``super`` to access the list[key] item
        if isinstance(key, int):
            return super(Choices, self).__getitem__(key)

        if not hasattr(self, key):
            raise KeyError("Attribute '%s' not found." % key)

        return getattr(self, key)

    def __repr__(self):
        """String representation of this ``Choices`` instance.

        Notes
        -----
        It will represent the data passed and store in ``self.entries``, not the data really
        stored in the base list object, which is in the format expected by django, ie a list of
        tuples with only value and display name.
        Here, we display everything.

        Example
        -------

        >>> Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        [('FOO', 1, 'foo'), ('BAR', 2, 'bar')]

        """

        return '%s' % self.entries

    def __eq__(self, other):
        """Override to allow comparison with a tuple of choices, not only a list.

        It also allow to compare with default django choices, ie (value, display name), or
        with the format of ``Choices``, ie (constant name, value, display_name).

        Example
        -------

        >>> MY_CHOICES = Choices(('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        >>> MY_CHOICES == [('FOO', 1, 'foo'), ('BAR', 2, 'bar')]
        True
        >>> MY_CHOICES == (('FOO', 1, 'foo'), ('BAR', 2, 'bar'))
        True
        >>> MY_CHOICES == [(1, 'foo'), (2, 'bar')]
        True
        >>> MY_CHOICES == ((1, 'foo'), (2, 'bar'))
        True

        """

        # Convert to list if it's a tuple.
        if isinstance(other, tuple):
            other = list(other)

        # Compare to the list of entries if the first element seems to have a constant
        # name as first entry.
        if other and len(other[0]) == 3:
            return self.entries == other

        return super(Choices, self).__eq__(other)

    # TODO: implement __iadd__ and __add__

    def __reduce__(self):
        """Reducer to make the auto-created classes picklable.

        Returns
        -------
        tuple
            A tuple as expected by pickle, to recreate the object when calling ``pickle.loads``:
            1. a callable to recreate the object
            2. a tuple with all positioned arguments expected by this callable

        """

        return (
            # Function to create a ``Choices`` instance
            create_choice,
            (
                # The ``Choices`` class, or a subclass, used to create the current instance
                self.__class__,
                # The list of choices
                [
                    (
                        entry.constant.original_value,
                        entry.value.original_value,
                        entry.display.original_value,
                    )
                    for entry in self.entries
                ],
                # The list of subsets
                [
                    (
                        # The name
                        subset_name,
                        # The list of constants to use in this subset
                        [
                            c.original_value
                            for c in getattr(self, subset_name).constants.keys()
                        ]
                    )
                    for subset_name in self.subsets
                ],
                # Extra kwargs to pass to ``__ini__``
                {
                    'dict_class': self.dict_class,
                    'mutable': self._mutable,
                }
            )
        )


class OrderedChoices(Choices):
    """Simple subclass of ``Choices`` using ``OrderedDict`` as ``dict_class``

    Example
    -------

    Start by declaring your ``Choices``:

    >>> ALIGNMENTS = OrderedChoices(
    ...     ('BAD', 10, 'bad'),
    ...     ('NEUTRAL', 20, 'neutral'),
    ...     ('CHAOTIC_GOOD', 30, 'chaotic good'),
    ...     ('GOOD', 40, 'good'),
    ... )

    >>> ALIGNMENTS.dict_class
    <class 'collections.OrderedDict'>

    >>> ALIGNMENTS.constants
    OrderedDict([('BAD', ('BAD', 10, 'bad')), ('NEUTRAL', ('NEUTRAL', 20, 'neutral')), ('CHAOTIC_GOOD', ('CHAOTIC_GOOD', 30, 'chaotic good')), ('GOOD', ('GOOD', 40, 'good'))])
    >>> ALIGNMENTS.values
    OrderedDict([(10, ('BAD', 10, 'bad')), (20, ('NEUTRAL', 20, 'neutral')), (30, ('CHAOTIC_GOOD', 30, 'chaotic good')), (40, ('GOOD', 40, 'good'))])
    >>> ALIGNMENTS.displays
    OrderedDict([('bad', ('BAD', 10, 'bad')), ('neutral', ('NEUTRAL', 20, 'neutral')), ('chaotic good', ('CHAOTIC_GOOD', 30, 'chaotic good')), ('good', ('GOOD', 40, 'good'))])

    """

    def __init__(self, *choices, **kwargs):

        # Class to use for dicts
        if 'dict_class' not in kwargs:
            kwargs['dict_class'] = OrderedDict

        super(OrderedChoices, self).__init__(*choices, **kwargs)



def create_choice(klass, choices, subsets, kwargs):
    """Create an instance of a ``Choices`` object.

    Parameters
    ----------
    klass : type
        The class to use to recreate the object.
    choices : list(tuple)
        A list of choices as expected by the ``__init__`` method of ``klass``.
    subsets : list(tuple)
        A tuple with an entry for each subset to create. Each entry is a list with two entries:
        - the name of the subsets
        - a list of the constants to use for this subset
    kwargs : dict
        Extra parameters expected on the ``__init__`` method of ``klass``.

    Returns
    -------
    Choices
        A new instance of ``Choices`` (or other class defined in ``klass``).

    """

    obj = klass(*choices, **kwargs)
    for subset in subsets:
        obj.add_subset(*subset)
    return obj


if __name__ == '__main__':
    # pylint: disable=wrong-import-position,wrong-import-order
    import doctest
    doctest.testmod(report=True)
    from . import helpers
    doctest.testmod(m=helpers, report=True)



#!/usr/bin/env python

# from __future__ import unicode_literals

from os.path import abspath, dirname, join
from setuptools import setup, find_packages
import sys


def read_relative_file(filename):
    """Returns contents of the given file, which path is supposed relative
    to this module."""
    with open(join(dirname(abspath(__file__)), filename)) as f:
        return f.read()

install_requires = ['future']
if sys.version_info < (2, 7):
    install_requires.append('unittest2')

setup(
    name="django-extended-choices",
    version="1.1.1",
    license="BSD",
    description="Little helper application to improve django choices"
    "(for fields)",
    long_description=read_relative_file('README.rst'),
    url="https://github.com/twidi/django-extended-choices",
    author='Stephane "Twidi" Angel',
    author_email="s.angel@twidi.com",
    install_requires=install_requires,
    packages=find_packages(),
    include_package_data=True,
    extras_require={
        'dev': ['django'],
        'makedoc': ['django', 'sphinx', 'sphinxcontrib-napoleon', 'sphinx_rtd_theme'],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Framework :: Django :: 1.4",
        "Framework :: Django :: 1.5",
        "Framework :: Django :: 1.6",
        "Framework :: Django :: 1.7",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "Framework :: Django :: 1.10",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)
